@@ -89,6 +89,12 @@ GenServer::GenServer () :
     add_opt ('l', "pidfile",         &m_pidfile);
 	add_opt ('S', "log-server",      &m_log_server);
 	add_opt ('c', "log-level",       &m_log_level);
+
+	/** Form a valid log server address
+	 */
+	char hn[64];
+	::gethostname (hn, sizeof (hn)-1);
+	m_log_server += hn;
 }
 
 //------------------------------------------------------------------------------
@@ -107,12 +113,25 @@ init (int* argc, char* argv [], const char* ht_)
 	 * Solaris x86 whole path is returned. 
 	 * Scan through the path and get the process name. 
      */
-    if ( strchr(cp,'/') ) {
+	if (strchr(cp, ASSA_DIR_SEPARATOR)) {
 		cp += strlen(argv[0]); // position at the end
-		while ( *cp-- != '/' ) ;
+		while (*cp-- != ASSA_DIR_SEPARATOR) {
+			;
+		}
 		cp += 2;
     }
-    m_cmdline_name = cp;		
+
+#if defined (WIN32)				// get rid of '.exe'
+	char* extidx = cp;
+	while (*extidx) {
+		if (*extidx == '.') {
+			*extidx = '\0';
+			break;
+		}
+		extidx++;
+	}
+#endif
+	m_cmdline_name = cp;		
 	
     if (!parse_args ((const char **)argv)) {
 		std::cerr << "Error in arguments: " << get_opt_error () << std::endl;
@@ -147,6 +166,7 @@ init (int* argc, char* argv [], const char* ht_)
 		m_port = m_proc_name;
     }
 
+#if !defined(WIN32)
     /** Setup signal handling. 
 	 *  Ignore SIGHUP, SIGPIPE, SIGCHLD, SIGCLD, SIGALRM by default.
 	 */
@@ -190,7 +210,9 @@ init (int* argc, char* argv [], const char* ht_)
 	 * to properly terminate GenServer process. 
      */
     m_sig_dispatcher.install ( SIGTERM, (EventHandler*) this );
-	
+
+#endif // !defined(WIN32)
+
     /** Initialize other internal stuff.
 	 */
     init_internals ();
@@ -202,10 +224,16 @@ init_internals ()
 {
     static const char self[] = "GenServer::init_internals";
 
-    /** Set standard configuration file name
+    /** Set standard configuration file name. 
+	 *  For POSIX systems, it is $HOME/.procname.
+	 *  For Win32, it is $cwd/procname.ini.
 	 */
+#if defined (WIN32)
+    m_default_config_file = this->get_cmdline_name () + ".ini";
+#else
     m_default_config_file = "$HOME/." + this->get_cmdline_name ();
 	m_default_config_file = Utils::strenv (m_default_config_file.c_str ());
+#endif
 
     /** Remove existing log file if requested. Unlinking /dev/null 
 		character device and replacing it with a regular file leads
@@ -227,9 +255,6 @@ init_internals ()
 	 *  --with-log-server="yes" which takes precedence over 
 	 *  --log-file=/path/to/log
 	 */
-	char hn[64];
-	::gethostname (hn, sizeof (hn)-1);
-	m_log_server += hn;
 
 	Log::set_app_name (get_proc_name ());
 
@@ -256,7 +281,7 @@ init_internals ()
 			m_pidfile = "~/." + m_proc_name + ".pid";
 		}
 		if (! m_pidfile_lock.lock (m_pidfile)) {
-			DL((ERROR,"Failed to lock PID file: %s\n",
+			DL((ASSAERR,"Failed to lock PID file: %s\n",
 				m_pidfile_lock.get_error_msg ()));
 			exit (1);
 		}
@@ -280,6 +305,9 @@ bool
 GenServer::
 become_daemon ()
 {
+#if defined(WIN32)
+    return true;
+#else
     Fork f (Fork::LEAVE_ALONE, Fork::IGNORE_STATUS);
 
     if (!f.isChild ()) {	// parent exits
@@ -320,6 +348,8 @@ become_daemon ()
 	}
 #endif
     return (true);
+    
+#endif  // defined(WIN32)
 }
 
 int 
