@@ -9,7 +9,12 @@
 //  License as published by the Free Software Foundation; either
 //  version 2 of the License, or (at your option) any later version.
 //------------------------------------------------------------------------------
-#include <sys/errno.h>
+#if defined(WIN32)
+#   include <errno.h>
+#   include <ws2tcpip.h>
+#else
+#   include <sys/errno.h>
+#endif
 
 #include "assa/MemDump.h"
 #include "assa/IPv4Socket.h"
@@ -38,14 +43,20 @@ open (const int domain_)
 	trace_with_mask("IPv4Socket::open",SOCKTRACE);
 	
 	m_type = domain_;
+
 	m_fd = ::socket(domain_, SOCK_STREAM, 0);
-	if (m_fd < 0) {
-		EL((ERROR,"OS::socket() error\n"));
+
+	if (!is_valid_handler (m_fd)) {
+		EL((ASSAERR,"OS::socket() error: m_fd = %d\n", m_fd));
 		setstate (Socket::failbit);
+		disable_handler (m_fd);
 		return (false);
 	}
+	DL ((SOCK,"domain = %d, m_fd = %d\n", domain_, m_fd));
+
 	clear ();
 	turnOptionOn (Socket::nonblocking);
+
 	return (true);
 }
 
@@ -55,14 +66,13 @@ close()
 {
 	trace_with_mask("IPv4Socket::close()",SOCKTRACE);
 
-	if (m_fd >= 0) {
+	if (is_valid_handler (m_fd)) {
 		DL((SOCK,"Closed FD: %d\n",m_fd));
 
 		/*--- Flush data in output stream buffer ---*/
 		flush ();
-		::close(m_fd);
+		close_handler(m_fd);
 		setstate (Socket::failbit);
-		m_fd = -1;
 
 		/*--- 
 		  Socket can be re-opened in the future.
@@ -83,15 +93,17 @@ connect (const Address& his_address_)
 {
 	trace_with_mask("IPv4Socket::connect()",SOCKTRACE);
 
-	if (m_fd < 0 && open (getDomain()) == false) {
+	if (!is_valid_handler (m_fd) && open (getDomain()) == false) {
 		return false;
 	}
 
 	int ret = ::connect (m_fd, 
 						 (SA*) his_address_.getAddress(),
 						 his_address_.getLength());
-	if (ret < 0) {
-		if (errno == EINPROGRESS) {	// is it ASYNC connect in progress?
+	if (ret < 0) 
+	{
+        int e = get_errno ();	// is ASYNC connect in progress?
+        if (e == EINPROGRESS || e == EWOULDBLOCK) {	
 			DL((SOCK,"FD: %d OS::connect() error\n",m_fd));
 		}
 		else {
@@ -99,7 +111,9 @@ connect (const Address& his_address_)
 		}
 		return (false);
 	}
+
 	clear ();
+
 	DL((SOCK,"Connection opened on FD: %d\n", m_fd));
 	return (true);
 }
@@ -110,7 +124,10 @@ bind (const Address& addr_)
 {
 	trace_with_mask("IPv4Socket::bind",SOCKTRACE);
 
-	// if UNIX domain, save the path
+#if !defined(WIN32)
+	/** 
+	 * If UNIX domain, save the path
+	 */
 	if ( getDomain() == AF_UNIX ) {
 		char* p = ((SA_UN *) addr_.getAddress())->sun_path;
 		m_path = new char[strlen(p)+1];
@@ -123,6 +140,8 @@ bind (const Address& addr_)
 			}
 		}
 	}
+#endif
+
 	/*---
 	  From Stevens, Ch 7.5 (p.196):
 	  "Set the SO_REUSEADDR socket option before calling bind(2)
@@ -141,45 +160,48 @@ bind (const Address& addr_)
 	return (true);
 }
 
+/**
+   Here's an interesting spot - because accept() suppose to work
+   both for INET and UNIX domain socket addresses, we have to
+   allocate enough space and pass exact size of the address type 
+   expected.
+   
+   Otherwise, if we use, for example, struct sockaddr_un as max.
+   and accept returns struct sockaddr_in, we can cast back to 
+   struct sockaddr_in, but internally address data members
+   are not guaranteed to be aligned correctly!!!
+*/
 IPv4Socket*
 IPv4Socket::
 accept ()
 {
-	/*
-	  Here's interesting spot - because accept() suppose to work
-	  both for INET and UNIX domain socket addresses, we have to
-	  allocate enough space and pass exact size of the address type 
-	  expected.
-	
-	  Otherwise, if we use, for example, struct sockaddr_un as max.
-	  possible, and accept returns struct sockaddr_in, we can cast
-	  back to struct sockaddr_in, but internally address data members
-	  are not guaranteed to be aligned correctly!!!
-	*/
 	trace_with_mask("IPv4Socket::accept",SOCKTRACE);
 
 	socklen_t length = 0;
-	int new_fd = -1;
-	SA* remote_address = NULL;
+	SA*       remote_address = NULL;
+	handler_t new_fd;
+
+	disable_handler (new_fd);
 
 	if ( getDomain() == AF_UNIX ) {
 		length = sizeof(struct sockaddr_in);
 		remote_address = (SA*) new SA_IN;
 	}
-	else { 
+	else   /* AF_INET */
+	{ 
 		remote_address = (SA*) new SA_UN;
 		length = sizeof(struct sockaddr_un);
 	}
 	memset(remote_address, 0, length);
 
-#ifndef __CYGWIN32__
+#if !defined (_CYGWIN32__)
 	new_fd = ::accept(m_fd, remote_address, &length);
 #else
 	new_fd = ::accept(m_fd, remote_address, (int*)&length);
 #endif
 	
-	if ( new_fd < 0 ) {
-		EL((ERROR,"OS::accept() failed\n"));
+	if (!is_valid_handler (new_fd)) {
+		EL((ASSAERR,"::accept() failed (new_fd=%d)\n", new_fd));
 		close();
 		return NULL;
 	}
@@ -190,9 +212,10 @@ accept ()
 		    inet_ntoa(sa_in->sin_addr), ntohs( sa_in->sin_port)));
 	}
 	else {
+#if !defined(WIN32)
 		SA_UN* sa_un = (SA_UN*) remote_address;
-		DL((SOCK,"Accepted new UNIX connection from %s\n",
-			 sa_un->sun_path));
+		DL((SOCK,"Accepted new UNIX connection from %s\n", sa_un->sun_path));
+#endif
 	}
 	delete remote_address;
 
@@ -212,7 +235,9 @@ read (char* packet_, const unsigned int size_)
 	register int sz = size_;
 	char* tmp = packet_;
 	
-	if (m_fd < 0) return -1;
+	if (!is_valid_handler (m_fd) < 0) {
+		return -1;
+	}
 
 	len = 0;
 	if (m_rdbuf->unbuffered ()) { 
@@ -232,18 +257,18 @@ read (char* packet_, const unsigned int size_)
 	else {
 		len = m_rdbuf->sgetn (tmp, sz);
 	}
-	if (len == -1) {
-		/*
-		  Non-blocking socket delivered partial packet.
-		*/
-		if (errno != EWOULDBLOCK) {
-			EL((ERROR,"::read(fd = %d) \n",m_fd));
+	if (len == -1) 
+	{
+		/** Non-blocking socket delivered partial packet.
+		 */
+        if (get_errno () != EWOULDBLOCK) {
+			EL((ASSAERR,"::read (fd=%d) failed.\n",m_fd));
 			setstate (Socket::failbit);
 		}
 		return len;
 	}
 	tmp += len;
-	sz -= len;
+	sz  -= len;
 
 	if ((size_ - sz) == 0) 
 	{
@@ -269,13 +294,16 @@ write(const char* packet_, const unsigned int size_)
 {
 	trace_with_mask("IPv4Socket::write()",SOCKTRACE);
 
-	register int ret = 0;
+	int ret = 0;
 
-	if (m_fd < 0) return -1;
+	if (!is_valid_handler (m_fd)) {
+		return -1;
+	}
 
-	if (m_rdbuf->unbuffered ()) {
-		register int wlen = size_;
-		register char* p = (char*) packet_;
+	if (m_rdbuf->unbuffered ()) 
+	{
+		int wlen = size_;
+		char* p = (char*) packet_;
 
 		while (wlen-- > 0) {
 			if (m_rdbuf->sputc (*p++) == EOF) {
@@ -284,8 +312,9 @@ write(const char* packet_, const unsigned int size_)
 		}
 		ret = p - packet_;
 	}
-	else
+	else {
 		ret = m_rdbuf->sputn ((char*) packet_, size_);
+	}
 
 	if (ret > 0) {
 		DL((SOCK,"<= FD: %d Wrote %d bytes (requested %d bytes)\n",
@@ -302,19 +331,19 @@ clone () const
 	const char self[] = "IPv4Socket::clone"; 
 	trace_with_mask(self,SOCKTRACE);
 
-	int nfd = dup (m_fd);
-
+	handler_t nfd = dup (m_fd);
 	IPv4Socket* s = new IPv4Socket (nfd);
 
 	DL((SOCK,"Original socket has %d bytes in its get_area\n",
 	    m_rdbuf->in_avail ()));
 
-	if (nfd < 0 || !good ()) {
+	if (!is_valid_handler (nfd) || !good ()) {
 		s->setstate (Socket::failbit);
 	}
 	else {
 		s->clear ();
 	}
+
 	return s;
 }
 
