@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              assa/Logger_Impl.cpp
 //------------------------------------------------------------------------------
-// $Id: Logger_Impl.cpp,v 1.4 2005/12/17 19:53:29 vlg Exp $
+// $Id: Logger_Impl.cpp,v 1.5 2006/07/20 02:30:54 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2001 by Vladislav Grinchenko
 //
@@ -14,10 +14,15 @@
 
 #include <iostream>
 #include <iomanip>
+#include <string.h>				// strerror(3)
 
 #include "assa/TimeVal.h"
 #include "assa/Logger_Impl.h"
 
+#if defined (WIN32)
+#  include <windows.h>			// for vsnprintf() bug
+#endif
+
 using namespace ASSA;
 
 char Logger_Impl::m_msgbuf [LOGGER_MAXLINE];
@@ -32,7 +37,7 @@ add_timestamp (ostream& sink_)
     if (timestamp_enabled ()) {
 		TimeVal tv = TimeVal::gettimeofday ();
 		tv.tz (m_tz);
-		sink_ << tv.fmtString ("%m/%d/%G %T") << '.';
+		sink_ << tv.fmtString ("%m/%d/%Y %H:%M:%S") << '.';
 		char oldfill = sink_.fill('0');
 		sink_ << std::setw (3) << (tv.msec () % 1000000)/1000 << ' ';
 		sink_.fill (oldfill);
@@ -75,21 +80,30 @@ indent_func_name (ostream&      sink_,
 
 char* 
 Logger_Impl::
-format_msg (size_t expected_sz_, 
+format_msg (size_t      expected_sz_, 
 			const char* fmt_, 
-			va_list vap_, 
-			bool& release_)
+			va_list     vap_, 
+			bool&       release_) // tell the caller it needs to release memory
 {
+	char* msg = m_msgbuf;		// Use internal buffer
+	int ret = 0;
+
 	release_ = false;
-	char* msg = m_msgbuf;
 	expected_sz_++;				// Expected size includes '\0'
 
-	if (expected_sz_ >= LOGGER_MAXLINE) {
+	if (expected_sz_ >= LOGGER_MAXLINE) { // Allocate temporary buffer
 		msg = new char [expected_sz_];
 		release_ = true;
 	}
-	if (::vsnprintf (msg, expected_sz_, fmt_, vap_) < 0) {
-		return NULL;
+
+	ret = ::vsnprintf (msg, expected_sz_, fmt_, vap_);
+#if NEVER
+	if (ret < 0) {
+		std::cout << "Logger_Impl: format_mg(expected_sz=" << expected_sz_
+				  << ")=-1 failed! errno=" << errno << " ("
+				  << strerror(errno) << "\n" << std::flush;
 	}
-	return msg;
+#endif
+
+	return (ret < 0 ? NULL : msg);
 }
