@@ -15,7 +15,11 @@
 #define REACTOR_H
 
 #include <sys/time.h>			// select(2)
-#include <sys/resource.h>		// getrlimit(2)
+#include <map>
+
+#if !defined(WIN32)
+#    include <sys/resource.h>		// getrlimit(2)
+#endif
 
 #include "assa/EventHandler.h"
 #include "assa/Singleton.h"
@@ -82,8 +86,8 @@ public:
 	    @return true if success, false if error
 	 */
 	bool registerIOHandler (EventHandler* eh_, 
-							int fd_, 
-							EventType et_ = RWE_EVENTS);
+							handler_t     fd_, 
+							EventType     et_ = RWE_EVENTS);
 
 	/** Remove Event handler from reactor for either all I/O events
 	    or timeout event or both. If et_ is TIMEOUT_EVENT, all timers
@@ -107,7 +111,7 @@ public:
 	    @param fd_ File descriptor
 	    @return true on success, false if fd_ is out of range
 	*/
-	bool removeIOHandler (int fd_);
+	bool removeIOHandler (handler_t fd_);
 
 	/// Main waiting loop that blocks indefinitely processing events. 
 	void waitForEvents (void);
@@ -147,6 +151,16 @@ private:
 	Reactor& operator= (const Reactor&); /// no cloning
 
 private:
+	typedef std::map<u_int, EventHandler*> Fd2Eh_Map_Type;
+	typedef Fd2Eh_Map_Type::iterator Fd2Eh_Map_Iter;
+
+private:
+    /// Adjust maxfdp1 in a portable way (win32 ignores masfd alltogether).
+	void adjust_maxfdp1 (handler_t fd_,
+						 handler_t rmax_, 
+						 handler_t wmax_, 
+						 handler_t emax_);
+
 	/// Handle error in select(2) loop appropriately. 
 	bool handleError (void);
 
@@ -168,8 +182,9 @@ private:
 	/** Call handler's callback and, if callback returns negative
 	    value, remove it from the Reactor.
 	*/
-	void dispatchHandler ( FdSet& mask_, EventHandler** fdSet_,
-			       EH_IO_Callback callback_);
+	void dispatchHandler ( FdSet&          mask_, 
+						   Fd2Eh_Map_Type& fdSet_,
+						   EH_IO_Callback  callback_);
 	
 	/** Calculate closest timeout. If TimerQueue is not empty,
 	    then return smallest of maxtimeout and first in the queue.
@@ -179,29 +194,31 @@ private:
 	*/
 	void calculateTimeout (TimeVal*& howlong_, TimeVal* maxwait_);
 
-	
 private:
 	/** Max number of open files per process. This is the soft
 	    limit enforced by the kernel. It can be obtained/manipulated
 	    from the shell with ulimit/limit utilities, but may not
 	    exceed the hard limit.
 	*/
-	int m_noFiles;
+	int m_fd_setsize;
 
-	/// Max file descriptor number plus 1
-	int m_maxfd;
+	/**
+	 * Max file descriptor number (in all sets) plus 1.
+	 * This value is ignored by WIN32 implementation of select()
+	 */
+	handler_t m_maxfd_plus1;
 
 	/// Flag that indicates whether Reactor is active or had been stopped.
 	bool m_active;
 
 	/// Event handlers awaiting on READ_EVENT
-	EventHandler** m_readSet;
+	Fd2Eh_Map_Type m_readSet;
 
 	/// Event handlers awaiting on WRITE_EVENT
-	EventHandler** m_writeSet;
+	Fd2Eh_Map_Type m_writeSet;
 
 	/// Event handlers awaiting on EXCEPT_EVENT
-	EventHandler** m_exceptSet;
+	Fd2Eh_Map_Type m_exceptSet;
 
 	/// Handlers to wait for event on
 	MaskSet m_waitSet;
@@ -212,13 +229,15 @@ private:
 	/// The queue of Timers.
 	TimerQueue m_tqueue;
 };
-/*******************************************************************************
- Inline functions
-*******************************************************************************/
+
+//-----------------------------------------------------------------------------
+// Inline functions
+//-----------------------------------------------------------------------------
 
 inline void Reactor::deactivate (void) {  m_active = false; }
 
 } // end namespace ASSA
 
 
+
 #endif /* REACTOR_H */  
