@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: Socket.cpp,v 1.10 2006/03/14 15:51:16 vlg Exp $
+// $Id: Socket.cpp,v 1.12 2006/07/26 00:27:32 vlg Exp $
 //------------------------------------------------------------------------------
 //                              Socket.C
 //------------------------------------------------------------------------------
@@ -20,8 +20,13 @@
 #endif
 
 #if defined (__NetBSD__)
-#include <sys/filio.h>
-#include <sys/ioctl.h>
+#  include <sys/filio.h>
+#  include <sys/ioctl.h>
+#endif
+
+#if defined (WIN32)
+#  define O_NONBLOCK    04000
+typedef unsigned int socklen_t;
 #endif
 
 #include "assa/Socket.h"
@@ -45,17 +50,23 @@ getBytesAvail (void) const
 	trace_with_mask("Socket::getBytesAvail",SOCKTRACE);
 	  
 	Socket* This = (Socket*) this;
-	int ba = 0;
-	int ret = ioctl (m_fd, FIONREAD, &ba);
+	u_long ba    = 0;
+	int ret      = 0;
+
+#if defined(WIN32)
+	ret = ioctlsocket (m_fd, FIONREAD, &ba);
+#else 
+	ret = ioctl (m_fd, FIONREAD, &ba);
+#endif
 
 	if (ret == -1) {
-		EL((ERROR,"ioctl(2) failed with ret: %d\n", ret));
+		EL((ASSAERR,"ioctl(2) failed with ret: %d\n", ret));
 		return ret;
 	}
 	ba += This->rdbuf ()->in_avail ();
 
-	DL((SOCKTRACE,"%d bytes available for reading\n", ba));
-	return ba;
+	DL((SOCKTRACE,"%ld bytes available for reading\n", ba));
+	return (int(ba));
 }
 
 Socket&
@@ -74,37 +85,99 @@ int
 Socket::
 set_option (int level_, int optname_, int val_)
 {
-	int ret = setsockopt (m_fd, level_, optname_, (const char*) &val_, 
-			      sizeof (val_));
-	if (ret < 0)
+	int ret = setsockopt (m_fd, 
+						  level_, 
+						  optname_, 
+						  (const char*) &val_, 
+						  sizeof (val_));
+	if (ret < 0) {
 		setstate (Socket::failbit);
+	}
 	return ret;
 }
 
+/** This is a private function used only by class Socket.
+	The only value for flags_ is O_NONBLOCK.
+*/
 int
 Socket::
-set_fd_options (int flags_)
+set_fd_options (long flags_)
 {
+	trace_with_mask("Socket::set_fd_options",SOCKTRACE);
 	int val;
-	if ((val = fcntl (m_fd, F_GETFL, 0)) < 0) {
+	int ret;
+
+#if defined (WIN32)
+
+	u_long set_nonblock = 1;
+	if ((val = ioctlsocket (m_fd, FIONBIO, &set_nonblock)) == 0) {
+		m_nonblocking = true;
+		return 0;
+	}
+	return -1;
+
+#else  // POSIX/UNIX
+
+	if ((val = ::fcntl (m_fd, F_GETFL, 0)) < 0) {
 		return -1;
 	}
 	val |= flags_;		// turn flags on
 
-	return (fcntl (m_fd, F_SETFL, val) < 0) ? -1 : 0;
+	DL ((SOCKTRACE,"Set flags fcntl(%d, %s)\n", m_fd, 
+		 decode_fcntl_flags (val).c_str ()));
+
+	ret = ::fcntl (m_fd, F_SETFL, val);
+
+	val = ::fcntl (m_fd, F_GETFL, 0);
+	DL ((SOCKTRACE,"Flags are set to %s via fcntl(25)\n",
+		 decode_fcntl_flags (val).c_str ()));
+
+	return (ret);
+
+#endif
 }
 
+/** This is a private function used only by class Socket.
+	The only value for flags_ is O_NONBLOCK - set the socket
+	back to the blocking mode.
+*/
+
 int
 Socket::
-clear_fd_options (int flags_)
+clear_fd_options (long flags_)
 {
-	int val;
-	if ((val = fcntl (m_fd, F_GETFL, 0)) < 0) {
+	trace_with_mask("Socket::clear_fd_options",SOCKTRACE);
+	long oldflags;
+	long newflags;
+	int ret;
+
+#if defined (WIN32)
+
+	u_long set_block = 0;
+	if ((ret = ioctlsocket (m_fd, FIONBIO, &set_block)) == 0) {
+		m_nonblocking = false;
+		return 0;
+	}
+	return -1;
+
+#else
+
+	if ((oldflags = ::fcntl (m_fd, F_GETFL, 0)) < 0) {
 		return -1;
 	}
-	val &= ~flags_;	// turn flags off
+	newflags = oldflags & ~flags_;	// clear flags
 
-	return (fcntl (m_fd, F_SETFL, val) < 0) ? -1 : 0;
+	DL ((SOCKTRACE,"Set flags fcntl(%d, %s)\n", m_fd, 
+		 decode_fcntl_flags (newflags).c_str ()));
+
+	ret = ::fcntl (m_fd, F_SETFL, newflags);
+
+	newflags = ::fcntl (m_fd, F_GETFL, 0);
+	DL ((SOCKTRACE,"Flags are set to %s via fcntl(%d)\n",
+		 decode_fcntl_flags (newflags).c_str ()));
+
+	return (ret);
+#endif
 }
 
 bool
@@ -113,9 +186,6 @@ turnOptionOn (opt_t opt_)
 {
 	trace_with_mask("Socket::turnOptionOn",SOCKTRACE);
 
-	if (blocking == opt_) 
-		return clear_fd_options (O_NONBLOCK);
-
 	if (nonblocking == opt_) 
 		return set_fd_options (O_NONBLOCK);
 
@@ -123,7 +193,7 @@ turnOptionOn (opt_t opt_)
 	if (reuseaddr == opt_) 
 		optname = SO_REUSEADDR;
 	else {
-		EL((ERROR,"Invalid socket option\n"));
+		EL((ASSAERR,"Invalid socket option\n"));
 		return false;
 	}
 	return set_option (SOL_SOCKET, optname, 1) == 0;
@@ -135,9 +205,6 @@ turnOptionOff (opt_t opt_)
 {
 	trace_with_mask("Socket::turnOptionOff",SOCKTRACE);
 
-	if (blocking == opt_) 
-		return set_fd_options (O_NONBLOCK);
-
 	if (nonblocking == opt_) 
 		return clear_fd_options (O_NONBLOCK);
 
@@ -145,7 +212,7 @@ turnOptionOff (opt_t opt_)
 	if (reuseaddr == opt_) 
 		optname = SO_REUSEADDR;
 	else {
-		EL((ERROR,"Invalid socket option\n"));
+		EL((ASSAERR,"Invalid socket option\n"));
 		return false;
 	}
 	return set_option (SOL_SOCKET, optname, 0) == 0;
@@ -158,18 +225,19 @@ setOption (opt_t opt_, int arg_)
 	trace_with_mask("Socket::setOption(,)",SOCKTRACE);
 	int optname;
 
-	if (nonblocking == opt_) 
+	if (nonblocking == opt_) {
 		return (arg_ == 1) ? set_fd_options (O_NONBLOCK)
 			: clear_fd_options (O_NONBLOCK);
+	}
 
-	if (blocking == opt_) 
-		return (arg_ == 1) ? clear_fd_options (O_NONBLOCK)
-			: set_fd_options (O_NONBLOCK);
-
-	if (rcvlowat == opt_)      optname = SO_RCVLOWAT;
-	else if (sndlowat == opt_) optname = SO_SNDLOWAT;
+	if (rcvlowat == opt_) {
+		optname = SO_RCVLOWAT;
+	}
+	else if (sndlowat == opt_) {
+		optname = SO_SNDLOWAT;
+	}
 	else {
-		EL((ERROR,"Invalid socket option\n"));
+		EL((ASSAERR,"Invalid socket option\n"));
 		return false;
 	}
 	return set_option (SOL_SOCKET, optname, arg_) == 0;
@@ -182,20 +250,18 @@ getOption (opt_t opt_) const
 	trace_with_mask("Socket::getOption",SOCKTRACE);
 	int optval = 0;
 
-	if (nonblocking == opt_) {
-		if ((optval = fcntl (m_fd, F_GETFL, 0)) < 0) {
+	if (nonblocking == opt_) 
+	{
+#if defined (WIN32)
+		return (m_nonblocking ? 1 : 0);
+#else
+		if ((optval = ::fcntl (m_fd, F_GETFL, 0)) < 0) {
 			return -1;
 		}
 		return ((optval & O_NONBLOCK) == O_NONBLOCK ? 1 : 0);
+#endif
 	}
 
-	if (blocking == opt_) {
-		if ((optval = fcntl (m_fd, F_GETFL, 0)) < 0) {
-			return -1;
-		}
-		return ((optval & O_NONBLOCK) == O_NONBLOCK ? 0 : 1);
-	}
-	
 	int optname;
 	int level = SOL_SOCKET;
 	bool bin = false;
@@ -214,14 +280,18 @@ getOption (opt_t opt_) const
 		bin = true;
 	}
 	else {
-		EL((ERROR,"Invalid socket option\n"));
+		EL((ASSAERR,"Invalid socket option\n"));
 		return (-1);
 	}
 
-#ifndef __CYGWIN32__
-	ret = getsockopt (m_fd, level, optname, (char*) &optval, &len);
-#else
+#if defined (__CYGWIN32__) || defined (WIN32)
+
 	ret = getsockopt (m_fd, level, optname, (char*) &optval, (int*)&len);
+
+#else  // posix/unix
+
+	ret = getsockopt (m_fd, level, optname, (char*) &optval, &len);
+
 #endif
 
 	if (ret < 0) {
@@ -637,3 +707,37 @@ is_little_endian ()
 	endian_u.v = 256;
 	return (endian_u.c [0] == 0);
 }
+
+string
+Socket::
+decode_fcntl_flags (long mask_)
+{
+	string answer;
+
+	if (mask_ & O_RDONLY) {
+		answer = "O_RDONLY|";	// 000000
+	}
+	if (mask_ & O_WRONLY) {
+		answer += "O_WRONLY|";	// 000001
+	}
+	if (mask_ & O_RDWR) {
+		answer += "O_RDWR|";	// 000002
+	}
+	if (mask_ & O_APPEND) {
+		answer += "O_APPEND|";	// 001000
+	}
+	if (mask_ & O_NONBLOCK) {
+		answer += "O_NONBLOCK|";// 004000
+	}
+	if (mask_ & O_SYNC) {
+		answer += "O_SYNC|";	// 010000
+	}
+
+	if (mask_ & O_ASYNC) {		// 020000
+		answer += "O_ASYNC|";
+	}
+
+	answer.erase (answer.end () - 1);
+
+	return answer;
+}
