@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                          sighands_test.cpp
 //------------------------------------------------------------------------------
-// $Id: sighands_test.cpp,v 1.8 2005/10/08 02:42:01 vlg Exp $
+// $Id: sighands_test.cpp,v 1.9 2006/07/20 02:30:56 vlg Exp $
 //------------------------------------------------------------------------------
 const char title [] =
 "                                                                           \n"\
@@ -41,12 +41,11 @@ const char title [] =
 "  Date:   Jul 12, 2000                                                     \n";
 //------------------------------------------------------------------------------
 
+
+
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/ipc.h>
-#include <sys/sem.h>
-#include <sys/wait.h>
 #include <stdio.h>
 
 #include <stdlib.h>
@@ -59,6 +58,12 @@ using std::list;
 using std::string;
 using namespace std;
 
+#if !defined (WIN32)
+
+#include <sys/ipc.h>
+#include <sys/sem.h>
+#include <sys/wait.h>
+
 #include "assa/GenServer.h"
 #include "assa/Fork.h"
 #include "assa/Assure.h"
@@ -638,6 +643,8 @@ Test::run_test5 ()
     else m_status = 1;
 }
 
+#endif	// !WIN32
+
 //------------------------------------------------------------------------------
 //   ||
 // End Class Test
@@ -646,6 +653,8 @@ Test::run_test5 ()
 
 int main (int argc, char* argv[])
 {
+#if !defined (WIN32)
+
     int rev = 0;
     Test& t = *Test::get_instance ();
     
@@ -659,4 +668,10 @@ int main (int argc, char* argv[])
     t.process_events ();
     
     exit (t.status ());
+
+#else
+
+	exit (0);
+
+#endif
 }
